# from dcp.estimator import Estimator
from dcp.storage import tar_and_upload
from dcp.job import create_job
from dcp.estimator import Estimator
import os


def test_fit_without_source_dir_entry_file():
    entry_file = os.path.join(
        os.path.dirname(__file__),
        'iris/source_dir/entry_file.py')
    estimator = Estimator(entry_file=entry_file, gpu_count=1)
    estimator.fit()


def get_real_path(path):
    return os.path.join(os.path.dirname(__file__), path)


def test_fit_with_source_dir():
    entry_file = 'main.py'
    source_dir = get_real_path('imagenet/source_dir')
    estimator = Estimator(entry_file=entry_file,
                          source_dir=source_dir,
                          gpu_count=1)
    estimator.fit()


def test_tpu_keras():
    entry_file = 'resnet50.py'
    source_dir = get_real_path('tpu/resnet50_keras')
    estimator = Estimator(entry_file=entry_file,
                          source_dir=source_dir,
                          gpu_count=0)
    estimator.fit()


# test_fit_without_source_dir_entry_file()
test_fit_with_source_dir()
# test_tpu_keras()



import os
import random
from time import localtime, strftime
from os.path import exists, isdir
from dcp.storage import get_source_files, tar_and_upload
from dcp.job import create_job, default_job_name, get_log


class Estimator:
    def __init__(self, entry_file, gpu_count, source_dir=None):
        exists(entry_file)
        if source_dir:
            isdir(source_dir)
        self.source_dir = source_dir
        self.entry_file = entry_file
        self.gpu_count = gpu_count

    def get_random_job_name(self, job_name):
        return '{0}-{1}-{2}'.format(job_name if job_name else default_job_name,
                                    strftime("%Y-%m-%d-%H-%M-%S", localtime()),
                                    random.randint(100, 999))

    def upload_source_files(self, job_name):
        source_files = get_source_files(self.entry_file, self.source_dir)
        tar_and_upload(source_files, job_name)

    def _entry_location(self, entry_file, source_dir):
        if not source_dir:
            return entry_file
        return os.path.join(source_dir, entry_file)

    def _get_entry_location(self, entry_file, source_dir):
        location = self._entry_location(entry_file, source_dir)
        if os.path.isabs(location):
            return location[1:]
        return location

    def fit(self, job_name=None):
        job_name = self.get_random_job_name(job_name)
        self.upload_source_files(job_name)
        entry_file_location = self._get_entry_location(
            entry_file=self.entry_file, source_dir=self.source_dir)
        create_job(job_name=job_name,
                   gpu_count=self.gpu_count,
                   entry_file_location=entry_file_location)
        get_log(job_name)



import os
from kubernetes import client, config


def is_gke_provider():
    kube_config_default_location = os.environ.get(
        'KUBECONFIG', '{}/.kube/config'.format(os.environ['HOME']))
    _, active_context = config.list_kube_config_contexts(
        kube_config_default_location)
    return active_context['name'].startswith('gke')


def file_system_protocol():
    return 'gs://' if is_gke_provider() else 's3://'



import os
import tarfile
import tempfile
import tensorflow as tf
from .utils import file_system_protocol


key_prefix = 'source'
default_bucket = 'dcp-bucket'
source_dir_tar_file_name = 'sourcedir.tar.gz'
default_namespace = 'default'


def _dst(job_name):
    return '{protocol}{bucket}/{key_prefix}/{job_name}/{source_file}'.format(
        protocol=file_system_protocol(),
        bucket=default_bucket,
        key_prefix=key_prefix,
        job_name=job_name,
        source_file=source_dir_tar_file_name)


def tar_and_upload(source_files, job_name):
    with tempfile.NamedTemporaryFile(delete=True) as f:
        with tarfile.open(mode='w:gz', fileobj=f) as t:
            t.add(source_files, recursive=True)
        f.seek(0)
        src = f.name
        dst = _dst(job_name=job_name)
        tf.gfile.Copy(src, dst)
    print('source code is uploaded to {}'.format(dst))


def get_source_files(entry_file, source_dir):
    if not source_dir:
        return entry_file
    return source_dir



import os
from glob import glob
import sys

from setuptools import setup, find_packages

required_packages = [
    'kubernetes',
    'urllib3<1.25'
]

setup(
    name="dcp-sdk",
    version='0.0.13',
    description="lsdl tensorflow package",
    packages=find_packages('src'),
    package_dir={'': 'src'},
    package_data={'': ['*.yaml']},
    author="dudaji",
    url="https://gitlab.com/dudaji/cloud/dcp-sdk.git",
    license="MIT",
    install_requires=required_packages)



import os
import urllib3
import yaml
import json
from time import sleep
from subprocess import Popen, PIPE, STDOUT
from kubernetes import client, config
from kubernetes.client.rest import ApiException
from .storage import default_bucket, key_prefix, source_dir_tar_file_name
from .utils import file_system_protocol, is_gke_provider


urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
default_job_name = 'dcp'
default_namespace = 'default'
config.load_kube_config()


def _get_job_command(job_name, entry_file_location):
    path = '{0}/{1}/{2}'.format(
        key_prefix,
        job_name,
        source_dir_tar_file_name)
    src = '{protocol}{bucket}/{path}'.format(
        protocol=file_system_protocol(),
        bucket=default_bucket,
        path=path)
    return 'dcp_run {} {}'.format(
        src, entry_file_location)


def interpolate_body(body, job_name, command, gpu_count):
    body['metadata']['name'] = job_name
    spec = body['spec']['template']['spec']
    containers = spec['containers']
    container = containers[0]
    container['command'] = ['/bin/sh', '-c', command]
    container['name'] = job_name
    if gpu_count > 0:
        container['resources']['limits']['nvidia.com/gpu'] = gpu_count
    body['spec']['template']['spec']['containers'][0] = container
    return body


def _get_tf_job_yaml_file_path(gpu_count):
    suffix = 'gke' if is_gke_provider() else 'dke'
    suffix = 'tpu' if suffix == 'gke' and gpu_count == 0 else suffix
    filename = 'tf_job_yaml/tf_job_{}.yaml'.format(suffix)
    return os.path.join(os.path.dirname(__file__), filename)


def _api_body(job_name, entry_file_location, gpu_count):
    file_path = _get_tf_job_yaml_file_path(gpu_count)
    with open(file_path) as f:
        job_template = yaml.safe_load(f)
    command = _get_job_command(job_name, entry_file_location)
    return interpolate_body(body=job_template,
                            command=command,
                            job_name=job_name,
                            gpu_count=gpu_count)


def create_job(job_name, entry_file_location, gpu_count):
    body = _api_body(job_name, entry_file_location, gpu_count)
    try:
        api_instance = client.BatchV1Api()
        api_instance.create_namespaced_job(
            "default", body, pretty=True)
        print('job {} is created'.format(job_name))
    except ApiException as e:
        print(e)


def _get_ready_state_pod(job_name):
    api_instance = client.CoreV1Api()
    try:
        res = api_instance.list_namespaced_pod(
            default_namespace,
            include_uninitialized=True,
            label_selector='job-name={}'.format(job_name))
        pod = res.items.pop()
        pod_name = pod.metadata.name
        for condition in pod.status.conditions:
            if condition.type == 'Ready':
                return pod_name
        return None

    except ApiException as e:
        print(e)


def _pod_log(name):
    api_instance = client.CoreV1Api()
    try:
        res = api_instance.read_namespaced_pod_log(
            name,
            default_namespace,
            follow=True,
            pretty=True,
            timestamps=True)
        print(res)
    except ApiException as e:
        print(e)


def get_log(job_name):
    try:
        ready_state_pod = None
        while not ready_state_pod:
            ready_state_pod = _get_ready_state_pod(job_name)
            print('.', end=' ')
            sleep(1)
        _pod_log(ready_state_pod)

    except ApiException as e:
        print(e)
